@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Philip Lafleur
  */
@@ -23,9 +23,11 @@
 
 #include "gegl-matrix.h"
 
+#define EPSILON 1e-6
 
 #if 0
-static void gegl_matrix3_debug (GeglMatrix3 *matrix)
+static void 
+gegl_matrix3_debug (GeglMatrix3 *matrix)
 {
   if (matrix)
     {
@@ -78,7 +80,7 @@ gegl_matrix3_equal (GeglMatrix3 *matrix1,
 
   for (y = 0; y < 3; y++)
     for (x = 0; x < 3; x++)
-      if (matrix1->coeff [y][x] != matrix2->coeff [y][x])
+      if (fabs (matrix1->coeff [y][x] - matrix2->coeff [y][x]) > EPSILON)
         return FALSE;
   return TRUE;
 }
@@ -110,6 +112,14 @@ gegl_matrix3_is_translate (GeglMatrix3 *matrix)
   return gegl_matrix3_is_identity (&copy);
 }
 
+gboolean
+gegl_matrix3_is_affine (GeglMatrix3 *matrix)
+{
+  return fabs (matrix->coeff[2][0])       <= EPSILON &&
+         fabs (matrix->coeff[2][1])       <= EPSILON &&
+         fabs (matrix->coeff[2][2] - 1.0) <= EPSILON;
+}
+
 void
 gegl_matrix3_copy_into (GeglMatrix3 *dst,
                         GeglMatrix3 *src)
@@ -197,37 +207,36 @@ gegl_matrix3_multiply (GeglMatrix3 *left,
 
 void
 gegl_matrix3_originate (GeglMatrix3 *matrix,
-                   gdouble x,
-                   gdouble y)
+                        gdouble      x,
+                        gdouble      y)
 {
-
   /* assumes last row is [0 0 1] (true for affine transforms) */
   matrix->coeff [0][2] = matrix->coeff [0][0] * (-x) +
-                  matrix->coeff [0][1] * (-y) +
-                  matrix->coeff [0][2] + x;
+                         matrix->coeff [0][1] * (-y) +
+                         matrix->coeff [0][2] + x;
   matrix->coeff [1][2] = matrix->coeff [1][0] * (-x) +
-                  matrix->coeff [1][1] * (-y) +
-                  matrix->coeff [1][2] + y;
+                         matrix->coeff [1][1] * (-y) +
+                         matrix->coeff [1][2] + y;
 }
 
 void
-gegl_matrix3_transform_point (GeglMatrix3  *matrix,
+gegl_matrix3_transform_point (GeglMatrix3 *matrix,
                               gdouble     *x,
                               gdouble     *y)
 {
   gdouble xp, yp, w;
-  
+
   w = (*x * matrix->coeff [2][0] + *y * matrix->coeff [2][1] + matrix->coeff [2][2]);
 
-  xp = (*x * matrix->coeff [0][0] + *y * matrix->coeff [0][1] + matrix->coeff [0][2]) /w;
-  yp = (*x * matrix->coeff [1][0] + *y * matrix->coeff [1][1] + matrix->coeff [1][2]) /w;
+  xp = (*x * matrix->coeff [0][0] + *y * matrix->coeff [0][1] + matrix->coeff [0][2]) / w;
+  yp = (*x * matrix->coeff [1][0] + *y * matrix->coeff [1][1] + matrix->coeff [1][2]) / w;
 
   *x = xp;
   *y = yp;
 }
 
 void
-gegl_matrix3_parse_string (GeglMatrix3  *matrix,
+gegl_matrix3_parse_string (GeglMatrix3 *matrix,
                            const gchar *string)
 {
   gegl_matrix3_identity (matrix);
@@ -238,12 +247,12 @@ gegl_matrix3_parse_string (GeglMatrix3  *matrix,
       gfloat b;
       if (!p) return;
       p++;
-      a = strtod(p, &p);
+      a = g_ascii_strtod (p, &p);
       if (!p) return;
       p = strchr (string, ',');
       if (!p) return;
       p++;
-      b = strtod (p, &p);
+      b = g_ascii_strtod (p, &p);
       if (!p) return;
 
       matrix->coeff [0][2] = a;
@@ -261,7 +270,7 @@ gegl_matrix3_parse_string (GeglMatrix3  *matrix,
       for (i=0;i<3;i++)
         for (j=0;j<3;j++)
           {
-            a = strtod(p, &p);
+            a = g_ascii_strtod(p, &p);
             matrix->coeff [j][i] = a;
             if (!p) return;
             p = strchr (p, ',');
@@ -271,20 +280,27 @@ gegl_matrix3_parse_string (GeglMatrix3  *matrix,
     }
 }
 
-gchar *gegl_matrix3_to_string (GeglMatrix3 *matrix)
+gchar *
+gegl_matrix3_to_string (GeglMatrix3 *matrix)
 {
-  gchar *res;
+  gchar   *res;
+  gchar    dstring[G_ASCII_DTOSTR_BUF_SIZE];
   GString *str = g_string_new ("matrix(");
-  gint i, j;
-  gint a=0;
+  gint     i, j;
+  gint     a = 0;
 
-  for (i=0;i<3;i++)
-    for (j=0;j<3;j++)
+  for (i = 0; i < 3; i++)
+    for (j = 0; j < 3; j++)
       {
-        if (a!=0)
+        if (a != 0)
           g_string_append (str, ",");
-        a=1;
-        g_string_append_printf (str, "%f", matrix->coeff[j][i]);
+        a = 1;
+        /* Do not use g_string_append_printf () or any other printf
+         * function because they are locale dependent.
+         */
+        g_ascii_dtostr (dstring, sizeof (dstring),
+                        matrix->coeff[j][i]);
+        g_string_append (str, dstring);
       }
   g_string_append (str, ")");
   res = str->str;
