@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include "property-types/gegl-paramspecs.h"
+#include "property-types/gegl-audio-fragment.h"
 
 #ifdef G_OS_WIN32
 #include <direct.h>
@@ -82,8 +83,139 @@ static void each_knot_rel (const GeglPathItem *path_node,
   g_string_append_printf (str, "%srel ", fstr);
 }
 
+
 void
-gegl_create_chain_argv (char      **ops,
+gegl_node_set_time (GeglNode   *node,
+                    gdouble     time)
+{
+  GeglNode *iter = NULL;
+  if (!node)
+    return;
+
+  if (gegl_node_has_pad (node, "input"))
+  {
+    iter = gegl_node_get_producer (node, "input", NULL);
+    if (iter)
+      gegl_node_set_time (iter, time);
+  }
+  if (gegl_node_has_pad (node, "aux"))
+  {
+    iter = gegl_node_get_producer (node, "aux", NULL);
+    if (iter)
+      gegl_node_set_time (iter, time);
+  }
+
+  {
+    gint i;
+    guint n_properties;
+    GParamSpec **properties;
+
+    properties = gegl_operation_list_properties (gegl_node_get_operation (
+                                                   node),
+                                                 &n_properties);
+    for (i = 0; i < n_properties; i++)
+      {
+        const gchar *property_name = g_param_spec_get_name (
+          properties[i]);
+        GType property_type = G_PARAM_SPEC_VALUE_TYPE (properties[i]);
+        char tmpbuf[1024];
+        GeglPath *anim_path = NULL;
+        GQuark anim_quark;//, rel_quark;
+        sprintf (tmpbuf, "%s-anim", property_name);
+        anim_quark = g_quark_from_string (tmpbuf);
+        anim_path = g_object_get_qdata (G_OBJECT (node), anim_quark);
+
+        if (property_type == G_TYPE_FLOAT)
+          {
+            if (anim_path)
+            {
+              gdouble y;
+              gegl_path_calc_y_for_x (anim_path, time, &y);
+              gegl_node_set (node, property_name, y, NULL);
+            }
+          }
+        else if (property_type == G_TYPE_DOUBLE)
+          {
+            if (anim_path)
+            {
+              gdouble y;
+              gegl_path_calc_y_for_x (anim_path, time, &y);
+              gegl_node_set (node, property_name, y, NULL);
+            }
+          }
+        else if (property_type == G_TYPE_INT)
+          {
+            if (anim_path)
+            {
+              gdouble y;
+              gegl_path_calc_y_for_x (anim_path, time, &y);
+              gegl_node_set (node, property_name, (int)y, NULL);
+            }
+          }
+        else if (property_type == G_TYPE_UINT)
+          {
+            if (anim_path)
+            {
+              gdouble y;
+              gegl_path_calc_y_for_x (anim_path, time, &y);
+              gegl_node_set (node, property_name, (guint32)y, NULL);
+            }
+          }
+        else if (property_type == G_TYPE_BOOLEAN)
+          {
+          }
+        else if (property_type == G_TYPE_STRING)
+          {
+          }
+        else if (g_type_is_a (property_type, G_TYPE_ENUM))
+          {
+          }
+        else if (property_type == GEGL_TYPE_COLOR)
+          {
+          }
+        else if (property_type == GEGL_TYPE_PATH)
+          {
+          }
+        else if (property_type == G_TYPE_POINTER &&
+                 GEGL_IS_PARAM_SPEC_FORMAT (properties[i]))
+          {
+          }
+        else
+          {
+          }
+
+      }
+    //g_free (properties);
+  }
+}
+
+
+#if 0
+static char *
+gegl_migrate_saturation_0_0_to_1_0 (const char  *input)
+{
+  //params_add_entry (params, "colorspace=CIE-lab");
+  //return TRUE;
+  return g_strdup (input);
+}
+
+
+
+
+gboolean
+gegl_migrate_api (GeglNode    *node,
+                  const char  *operation,
+                  char       **params)
+{
+  // if there is no opi entry, nothing to migrate
+  // if we find opi key.. look for valid migration
+
+  return TRUE;
+}
+#endif
+
+void
+gegl_create_chain_argv (char      **argv,
                         GeglNode   *start,
                         GeglNode   *proxy,
                         double      time,
@@ -93,7 +225,7 @@ gegl_create_chain_argv (char      **ops,
 {
   GeglNode   *iter[10] = {start, NULL};
   GeglNode   *new = NULL;
-  gchar     **arg = ops;
+  gchar     **arg = argv;
   int level = 0;
   char       *level_op[10];
   char       *level_pad[10];
@@ -103,6 +235,18 @@ gegl_create_chain_argv (char      **ops,
   GHashTable *ht = NULL;
   GeglPath   *path = NULL;
   GString    *string = NULL;
+  GeglNode **ret_sinkp = NULL;
+#if 0
+  char      *op_args[64]={NULL, };
+  int        n_op_args = 0;
+#endif
+
+  if (error && *error)
+  {
+    GeglNode **an = (void*)error;
+    ret_sinkp = (void*)*an;
+    *error = NULL;
+  }
 
   remove_in_betweens (start, proxy);
 
@@ -270,6 +414,7 @@ gegl_create_chain_argv (char      **ops,
                   {
                     /* should check for incompatibility rather than difference
                      */
+                    fprintf (stderr, "{%s}", value);
                     if (!g_str_equal (value,
                                       gegl_operation_get_op_version (level_op[
                                                                        level])))
@@ -541,7 +686,8 @@ gegl_create_chain_argv (char      **ops,
                                 gegl_node_set (iter[level], key, absolute_path, NULL);
                                 free (absolute_path);
                               }
-                            gegl_node_set (iter[level], key, value, NULL);
+                            else
+                              gegl_node_set (iter[level], key, value, NULL);
                           }
                       }
                     else if (g_type_is_a (target_type, G_TYPE_STRING))
@@ -559,29 +705,24 @@ gegl_create_chain_argv (char      **ops,
                           }
                         else
                           {
-                            /* warn, but try to get a valid nick out of the
-                               old-style value
-                             * name
-                             */
-                            gchar *nick;
-                            gchar *c;
-                            g_printerr (
-                              "gegl (param_set %s): enum %s has no value '%s'\n",
-                              key,
-                              g_type_name (target_type),
-                              value);
-                            nick = g_strdup (value);
-                            for (c = nick; *c; c++)
+                            if (error)
+                            {
+                              GString *str = g_string_new ("");
+
+                              g_string_append_printf (str,
+                              "unhandled enum value: %s\n", value);
+
+                              g_string_append_printf (str, "accepted values:");
+                              for (int i = 0; i < eclass->n_values; i++)
                               {
-                                *c = g_ascii_tolower (*c);
-                                if (*c == ' ')
-                                  *c = '-';
+                                g_string_append_printf (str, " %s", eclass->values[i].value_nick);
                               }
-                            evalue = g_enum_get_value_by_nick (eclass, nick);
-                            if (evalue)
-                              gegl_node_set (iter[level], key, evalue->value,
-                                             NULL);
-                            g_free (nick);
+
+
+                              *error = g_error_new_literal (
+                                  g_quark_from_static_string ( "gegl"), 0, str->str);
+                              g_string_free (str, TRUE);
+                            }
                           }
                       }
                     else
@@ -627,10 +768,17 @@ gegl_create_chain_argv (char      **ops,
                                                proxy), "operation",
                                              level_op[level], NULL);
 
-                  if (iter[level] && gegl_node_has_pad (new, "input"))
-                    gegl_node_link_many (iter[level], new, proxy, NULL);
+                  if (gegl_node_has_pad (new, "output"))
+                  {
+                    if (iter[level] && gegl_node_has_pad (new, "input"))
+                      gegl_node_link_many (iter[level], new, proxy, NULL);
+                    else
+                      gegl_node_link_many (new, proxy, NULL);
+                  }
                   else
-                    gegl_node_link_many (new, proxy, NULL);
+                  {
+                    gegl_node_link_many (iter[level], new, NULL);
+                  }
                   iter[level] = new;
                 }
               else if (error)
@@ -695,7 +843,17 @@ gegl_create_chain_argv (char      **ops,
 
   g_free (prop);
   g_hash_table_unref (ht);
-  gegl_node_link_many (iter[level], proxy, NULL);
+
+
+  if (gegl_node_has_pad (iter[level], "output"))
+    gegl_node_link_many (iter[level], proxy, NULL);
+  else
+  {
+    if (ret_sinkp)
+    {
+      *ret_sinkp = iter[level];
+    }
+  }
 }
 
 void
@@ -715,13 +873,16 @@ gegl_create_chain (const char *str, GeglNode *op_start, GeglNode *op_end,
     }
 }
 
-/* TODO: serialize keyframed properties */
 static gchar *
-gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
-                 GHashTable *ht, GeglSerializeFlag flags)
+gegl_serialize2 (GeglNode         *start,
+                 GeglNode         *end,
+                 const char       *basepath,
+                 GHashTable       *ht,
+                 GeglSerializeFlag flags)
 {
   char *ret = NULL;
   gboolean trim_defaults = flags & GEGL_SERIALIZE_TRIM_DEFAULTS;
+  gboolean bake_anim = flags & GEGL_SERIALIZE_BAKE_ANIM;
   GeglNode *iter;
 
   GString *str = g_string_new ("");
@@ -797,6 +958,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
             gint i;
             guint n_properties;
             GParamSpec **properties;
+            gboolean printed = FALSE;
 
             properties = gegl_operation_list_properties (gegl_node_get_operation (
                                                            iter),
@@ -809,30 +971,30 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                 const GValue*default_value = g_param_spec_get_default_value (
                   properties[i]);
                 char tmpbuf[1024];
-                GeglPath *path = NULL;
-                gboolean printed = FALSE;
+                GeglPath *anim_path = NULL;
                 char *rel_orig = NULL;
                 GQuark anim_quark, rel_quark;
-                sprintf (tmpbuf, "%s-anim", properties[i]->name);
+                sprintf (tmpbuf, "%s-anim", property_name);
                 anim_quark = g_quark_from_string (tmpbuf);
-                sprintf (tmpbuf, "%s-rel", properties[i]->name);
+                sprintf (tmpbuf, "%s-rel", property_name);
                 rel_quark = g_quark_from_string (tmpbuf);
-                path = g_object_get_qdata (G_OBJECT (iter), anim_quark);
+                if (!bake_anim)
+                  anim_path = g_object_get_qdata (G_OBJECT (iter), anim_quark);
                 rel_orig = g_object_get_qdata (G_OBJECT (iter), rel_quark);
 
                 if (property_type == G_TYPE_FLOAT)
                   {
                     gfloat defval = g_value_get_float (default_value);
                     gfloat value;
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
 
-                    if (path)
+                    if (anim_path)
                     {
                       g_string_append_printf (s2, " %s={ ", property_name);
                       if (rel_orig)
-                        gegl_path_foreach (path, each_knot_rel, s2);
+                        gegl_path_foreach (anim_path, each_knot_rel, s2);
                       else
-                        gegl_path_foreach (path, each_knot, s2);
+                        gegl_path_foreach (anim_path, each_knot, s2);
                       g_string_append_printf (s2, " } ");
                     }
                     else if (value != defval || (!trim_defaults))
@@ -852,13 +1014,13 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     gdouble value;
                     gegl_node_get (iter, property_name, &value, NULL);
 
-                    if (path)
+                    if (anim_path)
                     {
                       g_string_append_printf (s2, " %s={ ", property_name);
                       if (rel_orig)
-                        gegl_path_foreach (path, each_knot_rel, s2);
+                        gegl_path_foreach (anim_path, each_knot_rel, s2);
                       else
-                        gegl_path_foreach (path, each_knot, s2);
+                        gegl_path_foreach (anim_path, each_knot, s2);
                       g_string_append_printf (s2, " } ");
                     }
                     else if (value != defval || (!trim_defaults))
@@ -877,15 +1039,15 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     gint defval = g_value_get_int (default_value);
                     gint value;
                     gchar str[64];
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
 
-                    if (path)
+                    if (anim_path)
                     {
                       g_string_append_printf (s2, " %s={ ", property_name);
                       if (rel_orig)
-                        gegl_path_foreach (path, each_knot_rel, s2);
+                        gegl_path_foreach (anim_path, each_knot_rel, s2);
                       else
-                        gegl_path_foreach (path, each_knot, s2);
+                        gegl_path_foreach (anim_path, each_knot, s2);
                       g_string_append_printf (s2, " } ");
                     }
                     else if (value != defval || (!trim_defaults))
@@ -903,15 +1065,15 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     guint defval = g_value_get_uint (default_value);
                     guint value;
                     gchar str[64];
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
 
-                    if (path)
+                    if (anim_path)
                     {
                       g_string_append_printf (s2, " %s={ ", property_name);
                       if (rel_orig)
-                        gegl_path_foreach (path, each_knot_rel, s2);
+                        gegl_path_foreach (anim_path, each_knot_rel, s2);
                       else
-                        gegl_path_foreach (path, each_knot, s2);
+                        gegl_path_foreach (anim_path, each_knot, s2);
                       g_string_append_printf (s2, " } ");
                     }
                     else if (value != defval || (!trim_defaults))
@@ -928,7 +1090,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                   {
                     gboolean value;
                     gboolean defval = g_value_get_boolean (default_value);
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
                     if (value != defval || (!trim_defaults))
                       {
                         if (flags & GEGL_SERIALIZE_INDENT)
@@ -946,7 +1108,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                   {
                     gchar *value;
                     const gchar *defval = g_value_get_string (default_value);
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
                     if (!g_str_equal (defval, value) || (!trim_defaults))
                       {
                         if (flags & GEGL_SERIALIZE_INDENT)
@@ -963,7 +1125,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     gint defval = g_value_get_enum (default_value);
                     gint value;
 
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
                     if (value != defval || (!trim_defaults))
                       {
                         GEnumValue *evalue = g_enum_get_value (eclass, value);
@@ -980,7 +1142,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     GeglColor *defcolor = g_value_get_object (default_value);
                     gchar     *value;
                     gchar     *defvalue = NULL;
-                    gegl_node_get (iter, properties[i]->name, &color, NULL);
+                    gegl_node_get (iter, property_name, &color, NULL);
                     g_object_get (color, "string", &value, NULL);
                     if (defcolor)
                       {
@@ -1002,7 +1164,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                   {
                     gchar *svg_path;
                     GeglPath *path;
-                    gegl_node_get (iter, properties[i]->name, &path, NULL);
+                    gegl_node_get (iter, property_name, &path, NULL);
                     svg_path = gegl_path_to_string (path);
                     g_object_unref (path);
                     if (flags & GEGL_SERIALIZE_INDENT)
@@ -1017,7 +1179,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                   {
                     const Babl *format;
                     const gchar *value = "";
-                    gegl_node_get (iter, properties[i]->name, &format, NULL);
+                    gegl_node_get (iter, property_name, &format, NULL);
                     if (format)
                       value = babl_get_name (format);
                     if (value[0] || (!trim_defaults))
@@ -1029,27 +1191,31 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                         printed = TRUE;
                       }
                   }
+                else if (property_type == GEGL_TYPE_AUDIO_FRAGMENT)
+                  {
+                    /* ignore */
+                  }
                 else
                   {
                     g_warning (
                       "%s: serialization of %s properties not implemented",
                       property_name, g_type_name (property_type));
                   }
+              }
 
-                if (printed && (flags & GEGL_SERIALIZE_INDENT))
+              if (printed && (flags & GEGL_SERIALIZE_INDENT))
                   g_string_append_printf (s2, "\n");
 
-                {
-                  GeglNode *aux = gegl_node_get_producer (iter, "aux", NULL);
-                  if (aux)
-                    {
-                      char *str = gegl_serialize2 (NULL, aux, basepath, ht,
-                                                   flags);
-                      g_string_append_printf (s2, " aux=[ %s ]%s", str,
+              {
+                GeglNode *aux = gegl_node_get_producer (iter, "aux", NULL);
+                if (aux)
+                  {
+                    char *str = gegl_serialize2 (NULL, aux, basepath, ht,
+                                                 flags);
+                    g_string_append_printf (s2, " aux=[ %s ]%s", str,
                             (flags& GEGL_SERIALIZE_INDENT)?"\n":" ");
-                      g_free (str);
-                    }
-                }
+                    g_free (str);
+                  }
               }
           }
 
@@ -1092,6 +1258,7 @@ gegl_node_new_from_serialized (const gchar *chaindata,
 {
   GeglNode *ret;
   GeglNode *foo;
+  gdouble time = 0.0;
 
   ret = gegl_node_new ();
   gegl_node_set (ret, "operation", "gegl:nop", NULL);
@@ -1099,7 +1266,7 @@ gegl_node_new_from_serialized (const gchar *chaindata,
   gegl_node_set (foo, "operation", "gegl:nop", NULL);
 
   gegl_node_link_many (foo, ret, NULL);
-  gegl_create_chain (chaindata, foo, ret, 0, 1024, path_root, NULL);
+  gegl_create_chain (chaindata, foo, ret, time, 1024, path_root, NULL);
 
   return ret;
 }
