@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås
  */
@@ -25,16 +25,10 @@
 #include <limits.h>
 
 #include "gegl.h"
-#include "gegl-types-internal.h"
-#include "graph/gegl-node.h"
-#include "graph/gegl-pad.h"
-#include "operation/gegl-operation.h"
-#include "property-types/gegl-color.h"
-#include "property-types/gegl-curve.h"
-#include "property-types/gegl-path.h"
 #include "property-types/gegl-paramspecs.h"
 #include "gegl-instrument.h"
 #include "gegl-xml.h"
+#include "gegl-audio-fragment.h"
 
 #ifdef G_OS_WIN32
 #define realpath(a, b)    _fullpath (b, a, _MAX_PATH)
@@ -70,9 +64,10 @@ struct _ParseData
 /* Search a paired attribute name/value arrays for an entry with a specific
  * name, return the value or null if not found.
  */
-static const gchar *name2val (const gchar **attribute_names,
-                              const gchar **attribute_values,
-                              const gchar  *name)
+static const gchar *
+name2val (const gchar **attribute_names,
+          const gchar **attribute_values,
+          const gchar  *name)
 {
   while (*attribute_names)
     {
@@ -87,15 +82,21 @@ static const gchar *name2val (const gchar **attribute_names,
   return NULL;
 }
 
-static void param_set (ParseData   *pd,
-                       GeglNode    *new,
-                       const gchar *param_name,
-                       const gchar *param_value)
+static void 
+param_set (ParseData   *pd,
+           GeglNode    *new,
+           const gchar *param_name,
+           const gchar *param_value)
 {
   if (!strcmp (param_name, "name"))
     {
       g_object_set (new, param_name, param_value, NULL);
     }
+  else if (!strcmp (param_name, "opi"))
+    {
+      /* should check if it is compatible with runtime version of op..
+       */
+    }
   else if (!strcmp (param_name, "id"))
     {
       g_hash_table_insert (pd->ids, g_strdup (param_value), new);
@@ -108,17 +109,15 @@ static void param_set (ParseData   *pd,
   else if (strcmp (param_name, "operation") &&
            strcmp (param_name, "type"))
     {
-      GeglOperation *operation;
       GParamSpec    *paramspec;
 set_clone_prop_as_well:
 
-      operation = new->operation;
-      paramspec = g_object_class_find_property (G_OBJECT_GET_CLASS (operation), param_name);
+      paramspec = gegl_node_find_property (new, param_name);
 
       if (!paramspec)
         {
           g_warning ("property %s not found for %s",
-                     param_name, gegl_node_get_debug_name (new));
+                     param_name, gegl_node_get_operation (new));
         }
       else if (g_type_is_a (G_PARAM_SPEC_TYPE (paramspec),
                             GEGL_TYPE_PARAM_FILE_PATH))
@@ -150,8 +149,9 @@ set_clone_prop_as_well:
                 }
               else
                 {
-                  g_warning ("Unable to obtain absolute path for parameter %s\n",
-                             param_name);
+                  g_warning ("Unable to obtain absolute path for parameter %s\n", param_name);
+                  // Attempt to set raw value, useful for '-' meaning stdin
+                  gegl_node_set (new, param_name, param_value, NULL);
                 }
             }
         }
@@ -159,6 +159,10 @@ set_clone_prop_as_well:
         {
           gegl_node_set (new, param_name, atoi (param_value), NULL);
         }
+      else if (paramspec->value_type == G_TYPE_UINT)
+        {
+          gegl_node_set (new, param_name, (guint) strtoul (param_value, NULL, 10), NULL);
+        }
       else if (paramspec->value_type == G_TYPE_FLOAT ||
                paramspec->value_type == G_TYPE_DOUBLE)
         {
@@ -190,7 +194,33 @@ set_clone_prop_as_well:
         {
           GEnumClass *eclass = g_type_class_peek (paramspec->value_type);
           GEnumValue *evalue = g_enum_get_value_by_nick (eclass, param_value);
-          gegl_node_set (new, param_name, evalue->value, NULL);
+          if (evalue)
+            {
+              gegl_node_set (new, param_name, evalue->value, NULL);
+            }
+          else
+            {
+              /* warn, but try to get a valid nick out of the old-style
+               * value name
+               */
+              gchar *nick;
+              gchar *c;
+              g_printerr ("gegl-xml (param_set %s): enum %s has no value '%s'\n",
+                          paramspec->name,
+                          g_type_name (paramspec->value_type),
+                          param_value);
+              nick = g_strdup (param_value);
+              for (c = nick; *c; c++)
+                {
+                  *c = g_ascii_tolower (*c);
+                  if (*c == ' ')
+                    *c = '-';
+                }
+              evalue = g_enum_get_value_by_nick (eclass, nick);
+              if (evalue)
+                gegl_node_set (new, param_name, evalue->value, NULL);
+              g_free (nick);
+            }
         }
       else if (paramspec->value_type == GEGL_TYPE_COLOR)
         {
@@ -207,9 +237,7 @@ set_clone_prop_as_well:
           if (pd->curve)
             {
               gegl_node_set (new, param_name, pd->curve, NULL);
-
-              g_object_unref (pd->curve);
-              pd->curve = NULL;
+              g_clear_object (&pd->curve);
             }
         }
       else if (paramspec->value_type == GEGL_TYPE_PATH)
@@ -218,9 +246,19 @@ set_clone_prop_as_well:
           gegl_path_parse_string (path, param_value);
           gegl_node_set (new, param_name, path, NULL);
         }
+      else if (paramspec->value_type == G_TYPE_POINTER &&
+               GEGL_IS_PARAM_SPEC_FORMAT (paramspec))
+        {
+          const Babl *format = NULL;
+
+          if (strlen (param_value))
+            format = babl_format (param_value);
+
+          gegl_node_set (new, param_name, format, NULL);
+        }
       else
         {
-          g_warning ("operation desired unknown parapspec type for %s",
+          g_warning ("operation desired unknown paramspec type for %s",
                      param_name);
         }
     }
@@ -250,12 +288,13 @@ set_clone_prop_as_well:
     }                                                                   \
 }
 
-static void start_element (GMarkupParseContext *context,
-                           const gchar         *element_name,
-                           const gchar        **attribute_names,
-                           const gchar        **attribute_values,
-                           gpointer             user_data,
-                           GError             **error)
+static void 
+start_element (GMarkupParseContext *context,
+               const gchar         *element_name,
+               const gchar        **attribute_names,
+               const gchar        **attribute_values,
+               gpointer             user_data,
+               GError             **error)
 {
   const gchar **a  = attribute_names;
   const gchar **v  = attribute_values;
@@ -272,12 +311,16 @@ static void start_element (GMarkupParseContext *context,
       else
         {
         }
+
       pd->state  = STATE_TREE_NORMAL;
       pd->parent = g_list_prepend (pd->parent, new);
 
-      gegl_node_get_output_proxy (new, "output"); /* creates the pad if it doesn't exist */
       if (pd->iter)
-        gegl_node_connect_from (pd->iter, "input", new, "output");
+	{
+	  gegl_node_get_output_proxy (pd->iter, "output");
+	  gegl_node_connect_from (pd->iter, "input", new, "output");
+	}
+
       pd->iter = gegl_node_get_output_proxy (new, "output");
     }
   else if (!strcmp (element_name, "graph"))
@@ -392,8 +435,10 @@ static void start_element (GMarkupParseContext *context,
         }
       else
         {
-          if (pd->iter)
-            gegl_node_connect_from (pd->iter, "input", new, "output");
+          if (pd->iter && gegl_node_has_pad(new, "output"))
+	    {
+	      gegl_node_connect_from (pd->iter, "input", new, "output");
+	    }
         }
       pd->parent = g_list_prepend (pd->parent, new);
       pd->state  = STATE_TREE_FIRST_CHILD;
@@ -401,11 +446,12 @@ static void start_element (GMarkupParseContext *context,
     }
 }
 
-static void text (GMarkupParseContext *context,
-                  const gchar         *text,
-                  gsize                text_len,
-                  gpointer             user_data,
-                  GError             **error)
+static void 
+text (GMarkupParseContext *context,
+      const gchar         *text,
+      gsize                text_len,
+      gpointer             user_data,
+      GError             **error)
 {
   ParseData *pd = user_data;
 
@@ -416,10 +462,11 @@ static void text (GMarkupParseContext *context,
 }
 
 /* Called for close tags </foo> */
-static void end_element (GMarkupParseContext *context,
-                         const gchar         *element_name,
-                         gpointer             user_data,
-                         GError             **error)
+static void 
+end_element (GMarkupParseContext *context,
+             const gchar         *element_name,
+             gpointer             user_data,
+             GError             **error)
 {
   ParseData *pd = user_data;
 
@@ -431,7 +478,7 @@ static void end_element (GMarkupParseContext *context,
   else if (!strcmp (element_name, "tree") ||
            !strcmp (element_name, "layers"))
     {
-      if (gegl_node_get_pad (pd->iter, "input"))
+      if (gegl_node_get_producer (pd->iter, "input", NULL))
         {
           gegl_node_connect_from (pd->iter, "input",
                                   gegl_node_get_input_proxy (GEGL_NODE (pd->parent->data), "input"),
@@ -452,8 +499,7 @@ static void end_element (GMarkupParseContext *context,
     }
   else if (!strcmp (element_name, "param"))
     {
-      g_free (pd->param);
-      pd->param = NULL;
+      g_clear_pointer (&pd->param, g_free);
     }
   else if (!strcmp (element_name, "curve"))
     {
@@ -485,9 +531,10 @@ static void end_element (GMarkupParseContext *context,
 /* Called on error, including one set by other
  * methods in the vtable. The GError should not be freed.
  */
-static void error (GMarkupParseContext *context,
-                   GError              *error,
-                   gpointer             user_data)
+static void 
+error (GMarkupParseContext *context,
+       GError              *error,
+       gpointer             user_data)
 {
   gint  line_number;
   gint  char_number;
@@ -505,8 +552,9 @@ static GMarkupParser parser = {
   error
 };
 
-static void each_ref (gpointer value,
-                      gpointer user_data)
+static void 
+each_ref (gpointer value,
+          gpointer user_data)
 {
   ParseData *pd        = user_data;
   GeglNode  *dest_node = value;
@@ -519,16 +567,18 @@ static void each_ref (gpointer value,
   gegl_node_connect_from (dest_node, "input", source_node, "output");
 }
 
-GeglNode *gegl_node_new_from_xml (const gchar *xmldata,
-                                  const gchar *path_root)
+GeglNode *
+gegl_node_new_from_xml (const gchar *xmldata,
+                        const gchar *path_root)
 {
-  glong                time = gegl_ticks ();
   ParseData            pd   = { 0, };
   GMarkupParseContext *context;
   gboolean             success = FALSE;
 
   g_return_val_if_fail (xmldata != NULL, NULL);
 
+  GEGL_INSTRUMENT_START();
+
   pd.ids       = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   pd.refs      = NULL;
   pd.path_root = path_root;
@@ -546,11 +596,7 @@ GeglNode *gegl_node_new_from_xml (const gchar *xmldata,
     }
   else
     {
-      if (pd.gegl)
-        {
-          g_object_unref (pd.gegl);
-          pd.gegl = NULL;
-        }
+      g_clear_object (&pd.gegl);
     }
 
   g_list_free (pd.refs);
@@ -558,8 +604,7 @@ GeglNode *gegl_node_new_from_xml (const gchar *xmldata,
   g_markup_parse_context_free (context);
   g_hash_table_destroy (pd.ids);
 
-  time = gegl_ticks () - time;
-  gegl_instrument ("gegl", "gegl_parse_xml", time);
+  GEGL_INSTRUMENT_END ("gegl", "gegl_parse_xml");
 
   return success ? GEGL_NODE (pd.gegl) : NULL;
 }
@@ -822,6 +867,14 @@ serialize_properties (SerializeState *ss,
               g_snprintf (str, sizeof (str), "%i", value);
               xml_param (ss, indent + 2, properties[i]->name, str);
             }
+          else if (properties[i]->value_type == G_TYPE_UINT)
+            {
+              guint value;
+              gchar str[64];
+              gegl_node_get (node, properties[i]->name, &value, NULL);
+              g_snprintf (str, sizeof (str), "%u", value);
+              xml_param (ss, indent + 2, properties[i]->name, str);
+            }
           else if (properties[i]->value_type == G_TYPE_BOOLEAN)
             {
               gboolean value;
@@ -885,6 +938,24 @@ serialize_properties (SerializeState *ss,
 
               g_object_unref (path);
             }
+          else if (properties[i]->value_type == G_TYPE_POINTER &&
+                   GEGL_IS_PARAM_SPEC_FORMAT (properties[i]))
+            {
+              const Babl  *format;
+              const gchar *value;
+              gegl_node_get (node, properties[i]->name, &format, NULL);
+              if (format)
+                value = babl_get_name (format);
+              else
+                value = "";
+              xml_param (ss, indent + 2, properties[i]->name, value);
+            }
+          else if (properties[i]->value_type == GEGL_TYPE_AUDIO_FRAGMENT)
+            {
+            }
+          else if (properties[i]->value_type == GEGL_TYPE_BUFFER)
+            {
+            }
           else
             {
               g_warning ("%s: serialization of %s properties not implemented",
@@ -958,7 +1029,8 @@ serialize_layer (SerializeState *ss,
 static void
 add_stack (SerializeState *ss,
            gint            indent,
-           GeglNode       *head)
+           GeglNode       *head,
+           GeglNode       *tail)
 {
   /*ind; g_string_append (ss->buf, "<stack>\n");
      indent+=2;*/
@@ -966,18 +1038,15 @@ add_stack (SerializeState *ss,
   if (GEGL_IS_NODE (head))
     {
       GeglNode *iter = head;
+      gboolean last = FALSE;
 
       while (iter)
         {
-          GeglPad     *input, *aux;
           const gchar *id = NULL;
           gchar       *class;
           gegl_node_get (iter, "operation", &class, NULL);
 
-          input = gegl_node_get_pad (iter, "input");
-          aux   = gegl_node_get_pad (iter, "aux");
-
-          if (gegl_node_get_num_sinks (iter) > 1)
+          if (gegl_node_get_consumers (iter, "output", NULL, NULL) > 1)
             {
               const gchar *new_id = g_hash_table_lookup (ss->clones, iter);
               if (new_id)
@@ -1007,13 +1076,9 @@ add_stack (SerializeState *ss,
                 }
               else
                 {
-                  if (aux &&
-                      gegl_pad_get_connected_to (aux))
+                  GeglNode *source_node = gegl_node_get_producer (iter, "aux", NULL);
+                  if (source_node)
                     {
-                      GeglPad  *source_pad;
-                      GeglNode *source_node;
-                      source_pad  = gegl_pad_get_connected_to (aux);
-                      source_node = gegl_pad_get_node (source_pad);
                       {
                         GeglNode *graph = g_object_get_data (G_OBJECT (source_node),
                                                              "graph");
@@ -1035,13 +1100,17 @@ add_stack (SerializeState *ss,
                         xml_attr (ss->buf, "operation", class);
                         if (id != NULL)
                           xml_attr (ss->buf, "id", id);
+
+                        if (gegl_node_get_passthrough (iter) == TRUE)
+                          xml_attr (ss->buf, "passthrough", "true");
+
                         g_free (name);
                         g_free (class);
                       }
 
                       g_string_append (ss->buf, ">\n");
                       serialize_properties (ss, indent + 4, iter);
-                      add_stack (ss, indent + 4, source_node);
+                      add_stack (ss, indent + 4, source_node, NULL);
 
                       ind; g_string_append (ss->buf, "</node>\n");
                     }
@@ -1065,6 +1134,10 @@ add_stack (SerializeState *ss,
                             xml_attr (ss->buf, "operation", class);
                             if (id != NULL)
                               xml_attr (ss->buf, "id", id);
+
+                            if (gegl_node_get_passthrough (iter) == TRUE)
+                              xml_attr (ss->buf, "passthrough", "true");
+
                             g_free (name);
                             g_free (class);
                           }
@@ -1077,26 +1150,34 @@ add_stack (SerializeState *ss,
                 }
             }
           id = NULL;
-          if (input)
+
+          if (last)
+            iter = NULL;
+          else
             {
-              GeglPad *source_pad;
-              source_pad = gegl_pad_get_connected_to (input);
-              if (source_pad)
+              GeglNode *source_node = gegl_node_get_producer (iter, "input", NULL);
+              if (source_node)
                 {
-                  GeglNode *source_node = gegl_pad_get_node (source_pad);
                   GeglNode *graph       = g_object_get_data (G_OBJECT (source_node),
                                                              "graph");
+
+                  if (source_node == tail)
+                    last = TRUE;
+
+                  /* if source_node is a proxy then make it point to
+                   * the actual node
+                   */
                   if (graph)
                     source_node = graph;
+
+                  if (source_node == tail)
+                    last = TRUE;
+
                   iter = source_node;
                 }
               else
                 iter = NULL;
             }
-          else
-            {
-              iter = NULL;
-            }
 
           g_free (class);
         }
@@ -1106,9 +1187,11 @@ add_stack (SerializeState *ss,
 }
 
 gchar *
-gegl_node_to_xml (GeglNode    *gegl,
-                  const gchar *path_root)
+gegl_node_to_xml_full (GeglNode    *head,
+                       GeglNode    *tail,
+                       const gchar *path_root)
 {
+  GeglOperation *operation;
   SerializeState  ss;
 
   ss.buf         = g_string_new ("");
@@ -1117,12 +1200,27 @@ gegl_node_to_xml (GeglNode    *gegl,
   ss.clones      = g_hash_table_new (NULL, NULL);
   ss.terse       = FALSE;
 
-  gegl = gegl_node_get_output_proxy (gegl, "output");
+  operation = gegl_node_get_gegl_operation (head);
+  /* this case is for empty graphs, and graphs with nodes that are
+   * not meta-ops
+   */
+  if (!operation)
+    head = gegl_node_get_output_proxy (head, "output");
+
+  if (tail)
+    {
+      operation = gegl_node_get_gegl_operation (tail);
+      /* this case is for empty graphs, and graphs with nodes that are
+       * not meta-ops
+       */
+      if (!operation)
+        tail = gegl_node_get_input_proxy (tail, "input");
+    }
 
   g_string_append (ss.buf, "<?xml version='1.0' encoding='UTF-8'?>\n");
   g_string_append (ss.buf, "<gegl>\n");
 
-  add_stack (&ss, 2, gegl);
+  add_stack (&ss, 2, head, tail);
 
   g_string_append (ss.buf, "</gegl>\n");
 
@@ -1131,3 +1229,13 @@ gegl_node_to_xml (GeglNode    *gegl,
 
   return g_string_free (ss.buf, FALSE);
 }
+
+gchar *
+gegl_node_to_xml (GeglNode    *gegl,
+                  const gchar *path_root)
+{
+  gchar *xml;
+
+  xml = gegl_node_to_xml_full (gegl, NULL, path_root);
+  return xml;
+}
