@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2011 Michael Muré <batolettre@gmail.com>
  * Copyright 2008 Øyvind Kolås <pippin@gimp.org>
@@ -25,7 +25,6 @@
 
 #include "gegl.h"
 #include "gegl-path.h"
-#include "gegl-utils.h"
 #include <glib/gprintf.h>
 
 #include <stdarg.h>
@@ -54,15 +53,17 @@ struct _GeglPathPrivate
 
   GeglRectangle dirtied;
   GeglRectangle cached_extent;
+  gboolean      cached_extent_empty;
   GeglMatrix3   matrix;
   gint frozen;
 };
 
 typedef struct _GeglPathPrivate  GeglPathPrivate;
 
-G_DEFINE_TYPE (GeglPath, gegl_path, G_TYPE_OBJECT)
-#define GEGL_PATH_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o),\
-                                   GEGL_TYPE_PATH, GeglPathPrivate))
+G_DEFINE_TYPE_WITH_PRIVATE (GeglPath, gegl_path, G_TYPE_OBJECT)
+
+#define GEGL_PATH_GET_PRIVATE(o) \
+  ((GeglPathPrivate *) gegl_path_get_instance_private ((GeglPath *) (o)))
 
 enum
 {
@@ -189,6 +190,10 @@ gegl_path_init (GeglPath *self)
 {
   GeglPathPrivate *priv;
   priv = GEGL_PATH_GET_PRIVATE (self);
+  priv->flat_path_clean = FALSE;
+  priv->length_clean = FALSE;
+  priv->calc_clean = FALSE;
+  priv->cached_extent_empty = TRUE;
   gegl_matrix3_identity (&priv->matrix);
 }
 
@@ -201,8 +206,6 @@ gegl_path_class_init (GeglPathClass *klass)
   gobject_class->set_property = set_property;
   gobject_class->get_property = get_property;
 
-  g_type_class_add_private (klass, sizeof (GeglPathPrivate));
-
   gegl_path_signals[GEGL_PATH_CHANGED] =
     g_signal_new ("changed", G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
@@ -261,15 +264,8 @@ get_property (GObject    *gobject,
 GeglPath *
 gegl_path_new (void)
 {
-  GeglPath        *self = GEGL_PATH (g_object_new (GEGL_TYPE_PATH, NULL));
-  GeglPathPrivate *priv = GEGL_PATH_GET_PRIVATE (self);
-
-  gegl_path_init (self);
-  priv->flat_path_clean = FALSE;
-  priv->length_clean = FALSE;
-  priv->calc_clean = FALSE;
-
-  return self;
+  return g_object_new (GEGL_TYPE_PATH,
+                       NULL);
 }
 
 GeglPath *
@@ -284,7 +280,7 @@ gboolean
 gegl_path_is_empty (GeglPath *path)
 {
   GeglPathPrivate *priv = GEGL_PATH_GET_PRIVATE (path);
-  return priv->path != NULL;
+  return priv->path == NULL;
 }
 
 gint
@@ -523,7 +519,7 @@ gegl_path_calc (GeglPath   *self,
                 gdouble    *yd)
 {
   GeglPathPrivate *priv = GEGL_PATH_GET_PRIVATE (self);
-  GeglPathList *entry = priv->flat_path;
+  GeglPathList *entry;
   GeglPathList *stop  = NULL;
   gdouble rel_pos  = 0.0;
   gdouble leftover = 0.0;
@@ -532,6 +528,7 @@ gegl_path_calc (GeglPath   *self,
   if (!self)
     return FALSE;
   ensure_flattened (self);
+  entry = priv->flat_path;
 
   if (priv->calc_clean && (pos > priv->calc_leftover))
     {
@@ -580,13 +577,21 @@ gegl_path_get_bounds (GeglPath *self,
                       gdouble  *min_y,
                       gdouble  *max_y)
 {
+  gdouble dummy;
   GeglPathPrivate *priv;
-  GeglPathList *iter;
+  GeglPathList    *iter;
+  gboolean         first_point = TRUE;
 
-  *min_x = 256.0;
-  *min_y = 256.0;
-  *max_x = -256.0;
-  *max_y = -256.0;
+  /* protecting against NULL arguments */
+  if (!min_x) min_x = &dummy;
+  if (!max_x) max_x = &dummy;
+  if (!min_y) min_y = &dummy;
+  if (!max_y) max_y = &dummy;
+
+  *min_x = 0.0;
+  *min_y = 0.0;
+  *max_x = 0.0;
+  *max_y = 0.0;
 
   if (!self)
     return;
@@ -612,15 +617,16 @@ gegl_path_get_bounds (GeglPath *self,
 
       for (i=0;i<max;i++)
         {
-          if (iter->d.point[i].x < *min_x)
+          if (iter->d.point[i].x < *min_x || first_point)
             *min_x = iter->d.point[i].x;
-          if (iter->d.point[i].x > *max_x)
+          if (iter->d.point[i].x > *max_x || first_point)
             *max_x = iter->d.point[i].x;
-          if (iter->d.point[i].y < *min_y)
+          if (iter->d.point[i].y < *min_y || first_point)
             *min_y = iter->d.point[i].y;
-          if (iter->d.point[i].y > *max_y)
+          if (iter->d.point[i].y > *max_y || first_point)
             *max_y = iter->d.point[i].y;
 
+          first_point = FALSE;
         }
       iter=iter->next;
     }
@@ -786,6 +792,15 @@ gegl_path_remove_node (GeglPath *vector,
   gegl_path_emit_changed (vector, NULL);
 }
 
+void
+gegl_path_dirty (GeglPath *path)
+{
+  GeglPathPrivate *priv = GEGL_PATH_GET_PRIVATE (path);
+  priv->flat_path_clean = FALSE;
+  priv->length_clean = FALSE;
+  gegl_path_emit_changed (path, NULL);
+}
+
 void
 gegl_path_parse_string (GeglPath    *vector,
                         const gchar *path)
@@ -847,11 +862,7 @@ gegl_path_parse_string (GeglPath    *vector,
         p++;
     }
 
-  priv->flat_path_clean = FALSE;
-  priv->length_clean = FALSE;
-  {
-   gegl_path_emit_changed (vector, NULL);
-  }
+  gegl_path_dirty (vector);
 }
 
 void
@@ -897,7 +908,7 @@ gegl_path_append (GeglPath *self,
 
       x0 = iter->d.point[0].x;
       y0 = iter->d.point[0].y;
-      for (iter2=priv->path;iter2 && iter2->next != iter;iter2=iter2->next);
+      for (iter2=priv->path;iter2 && iter2->next != iter && iter2->next;iter2=iter2->next);
       x1 = iter2->d.point[0].x;
       y1 = iter2->d.point[0].y;
 
@@ -1041,23 +1052,55 @@ gegl_path_emit_changed (GeglPath            *self,
   if (priv->frozen)
     return;
 
-  gegl_path_get_bounds (self, &min_x, &max_x, &min_y, &max_y);
-
-  rect.x = floor (min_x);
-  rect.y = floor (min_y);
-  rect.width = ceil (max_x) - floor (min_x);
-  rect.height = ceil (max_y) - floor (min_y);
-
-  temp = priv->cached_extent;
-  priv->cached_extent = rect;
-
-  if (!bounds)
+  if (! gegl_path_is_empty (self))
     {
-       gegl_rectangle_bounding_box (&temp, &temp, &rect);
-       bounds = &temp;
+      gegl_path_get_bounds (self, &min_x, &max_x, &min_y, &max_y);
+
+      rect.x = floor (min_x);
+      rect.y = floor (min_y);
+      rect.width = ceil (max_x) - floor (min_x);
+      rect.height = ceil (max_y) - floor (min_y);
+
+      if (!bounds)
+        {
+          if (! priv->cached_extent_empty)
+            {
+              temp.x      = MIN (rect.x, priv->cached_extent.x);
+              temp.y      = MIN (rect.y, priv->cached_extent.y);
+              temp.width  = MAX (rect.x                +
+                                 rect.width,
+                                 priv->cached_extent.x +
+                                 priv->cached_extent.width)  - temp.x;
+              temp.height = MAX (rect.y                +
+                                 rect.height,
+                                 priv->cached_extent.y +
+                                 priv->cached_extent.height) - temp.y;
+              bounds = &temp;
+            }
+          else
+            {
+              bounds = &rect;
+            }
+        }
+
+      priv->cached_extent = rect;
+    }
+  else if (! priv->cached_extent_empty)
+    {
+      if (!bounds)
+        {
+          temp = priv->cached_extent;
+          bounds = &temp;
+        }
+    }
+
+  priv->cached_extent_empty = gegl_path_is_empty (self);
+
+  if (bounds)
+    {
+      g_signal_emit (self, gegl_path_signals[GEGL_PATH_CHANGED], 0,
+                     bounds, NULL);
     }
-  g_signal_emit (self, gegl_path_signals[GEGL_PATH_CHANGED], 0,
-                 bounds, NULL);
 }
 
 
@@ -1075,12 +1118,13 @@ ensure_flattened (GeglPath *vector)
   gint i;
   GeglPathList *path = priv->path;
   GeglPathList *new_path;
+  GeglPathList *old_path = NULL;
   GeglPathClass *klass= GEGL_PATH_GET_CLASS (vector);
 
   if (priv->flat_path_clean)
     return;
   if (priv->flat_path)
-    gegl_path_list_destroy (priv->flat_path);
+    old_path = priv->flat_path;
 
   for (i=0;klass->flattener[i];i++)
     {
@@ -1094,6 +1138,8 @@ ensure_flattened (GeglPath *vector)
     }
 
   priv->flat_path = gegl_path_list_flatten (&priv->matrix, path);
+  if (old_path)
+    gegl_path_list_destroy (old_path);
   if (path != priv->path)
     gegl_path_list_destroy (path);
   priv->flat_path_clean = TRUE;
@@ -1621,6 +1667,51 @@ gegl_path_point_dist (GeglPathPoint *a,
                (a->y-b->y)*(a->y-b->y));
 }
 
+#define Y_FOR_X_RES 1024
+
+gint gegl_path_calc_y_for_x (GeglPath *path,
+                             gdouble   x,
+                             gdouble  *y)
+{
+  gdouble xs[Y_FOR_X_RES];
+  gdouble ys[Y_FOR_X_RES];
+  gdouble best_xd = 4096.0;
+  gdouble second_best_xd = 4096.0;
+  double sum_xd;
+  gint best = 0;
+  gint second_best = 0;
+
+  int i;
+  if (!y)
+    return -1;
+  gegl_path_calc_values (path, Y_FOR_X_RES, xs, ys);
+  for (i = 0; i < Y_FOR_X_RES; i++)
+  {
+    if (fabs (xs[i] - x) < best_xd)
+    {
+      second_best_xd = best_xd;
+      best_xd = fabs (xs[i] - x);
+      second_best = best;
+      best = i;
+    }
+
+  }
+
+   sum_xd  = best_xd + second_best_xd;
+
+   if (best_xd < 0.0001)
+   {
+     *y = ys[best];
+   }
+   else
+   {
+      *y = ys[best] * (1.0 - (best_xd-sum_xd)/sum_xd) +
+           ys[second_best] * ((best_xd-sum_xd)/sum_xd);
+   }
+
+  return 0;
+}
+
 /* --------------------------------------------------------------------------
  * A GParamSpec class to describe behavior of GeglPath as an object property
  * follows.
