@@ -11,30 +11,33 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  *
  *
  * Copyright 2006 Dominik Ernst <dernst@gmx.de>
  */
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_string (path, "File", "", "Path of file to load.")
+property_file_path (path, "File", "")
+  description ("Path of file to load.")
 
 #else
 
-#define GEGL_CHANT_TYPE_SOURCE
-#define GEGL_CHANT_C_FILE       "exr-load.cpp"
+#define GEGL_OP_SOURCE
+#define GEGL_OP_NAME exr_load
+#define GEGL_OP_C_FILE       "exr-load.cpp"
 
 extern "C" {
-#include "gegl-chant.h"
+#include "gegl-op.h"
 }
 
 #include <ImfInputFile.h>
 #include <ImfChannelList.h>
 #include <ImfRgbaFile.h>
 #include <ImfRgbaYca.h>
+#include <ImfChromaticities.h>
 #include <ImfStandardAttributes.h>
 
 #include <stdio.h>
@@ -408,30 +411,39 @@ insert_channels (FrameBuffer  &fb,
                  gint          format_flags,
                  gint          bpp)
 {
-  gint alpha_offset = 12;
+  gint alpha_offset;
   PixelType tp;
+  // bytes per channel - is there a babl mechanism to obtain that from format_flags?
+  gint bpc = 4;
 
   if (format_flags & COLOR_U32)
     tp = UINT;
+  else if (format_flags & COLOR_FP16)
+    {
+      tp = HALF;
+      bpc = 2;
+    }
   else
     tp = FLOAT;
 
+  alpha_offset = bpc*3;
+
   if (format_flags & COLOR_RGB)
     {
-      fb.insert ("R", Slice (tp, base,    bpp, 0, 1,1, 0.0));
-      fb.insert ("G", Slice (tp, base+4,  bpp, 0, 1,1, 0.0));
-      fb.insert ("B", Slice (tp, base+8,  bpp, 0, 1,1, 0.0));
+      fb.insert ("R", Slice (tp, base,          bpp, 0, 1,1, 0.0));
+      fb.insert ("G", Slice (tp, base+bpc,      bpp, 0, 1,1, 0.0));
+      fb.insert ("B", Slice (tp, base+bpc*2,    bpp, 0, 1,1, 0.0));
     }
   else if (format_flags & COLOR_C)
     {
-      fb.insert ("Y",  Slice (tp, base,   bpp,   0, 1,1, 0.5));
-      fb.insert ("RY", Slice (tp, base+4, bpp*2, 0, 2,2, 0.0));
-      fb.insert ("BY", Slice (tp, base+8, bpp*2, 0, 2,2, 0.0));
+      fb.insert ("Y",  Slice (tp, base,         bpp,   0, 1,1, 0.5));
+      fb.insert ("RY", Slice (tp, base+bpc,     bpp*2, 0, 2,2, 0.0));
+      fb.insert ("BY", Slice (tp, base+bpc*2,   bpp*2, 0, 2,2, 0.0));
     }
   else if (format_flags & COLOR_Y)
     {
       fb.insert ("Y",  Slice (tp, base, bpp, 0, 1,1, 0.5));
-      alpha_offset = 4;
+      alpha_offset = bpc;
     }
 
   if (format_flags & COLOR_ALPHA)
@@ -527,6 +539,7 @@ query_exr (const gchar *path,
 {
   gchar format_string[16];
   gint format_flags = 0;
+  const Babl *space = NULL;
 
   try
     {
@@ -539,10 +552,26 @@ query_exr (const gchar *path,
       *width  = dw.max.x - dw.min.x + 1;
       *height = dw.max.y - dw.min.y + 1;
 
+      if (hasChromaticities(file.header()))
+      {
+        const Chromaticities &c2 = chromaticities (file.header());
+        space = babl_space_from_chromaticities
+ (NULL, c2.white[0], c2.white[1], c2.red[0], c2.red[1], c2.green[0], c2.green[1], c2.blue[0], c2.blue[1], babl_trc ("sRGB"), babl_trc ("sRGB"), babl_trc ("sRGB"), BABL_SPACE_FLAG_EQUALIZE);
+      }
+
+      if (ch.findChannel ("A"))
+        {
+          format_flags |= COLOR_ALPHA;
+        }
+
+
       if (ch.findChannel ("R") || ch.findChannel ("G") || ch.findChannel ("B"))
         {
-          strcpy (format_string, "RGB");
-          format_flags = COLOR_RGB;
+          if (format_flags & COLOR_ALPHA)
+            strcpy (format_string, "RaGaBa");
+          else
+            strcpy (format_string, "RGB");
+          format_flags |= COLOR_RGB;
 
           if ((chan = ch.findChannel ("R")))
             pt = chan->type;
@@ -554,15 +583,21 @@ query_exr (const gchar *path,
       else if (ch.findChannel ("Y") &&
                (ch.findChannel("RY") || ch.findChannel("BY")))
         {
-          strcpy (format_string, "RGB");
-          format_flags = COLOR_Y | COLOR_C;
+          if (format_flags & COLOR_ALPHA)
+            strcpy (format_string, "RaGaBa");
+          else
+            strcpy (format_string, "RGB");
+          format_flags |= COLOR_Y | COLOR_C;
 
           pt = ch.findChannel ("Y")->type;
         }
       else if (ch.findChannel ("Y"))
         {
-          strcpy (format_string, "Y");
-          format_flags = COLOR_Y;
+          if (format_flags & COLOR_ALPHA)
+            strcpy (format_string, "Ya");
+          else
+            strcpy (format_string, "Y");
+          format_flags |= COLOR_Y;
           pt = ch.findChannel ("Y")->type;
         }
       else
@@ -571,11 +606,8 @@ query_exr (const gchar *path,
           return FALSE;
         }
 
-      if (ch.findChannel ("A"))
-        {
-          strcat (format_string, "A");
-          format_flags |= COLOR_ALPHA;
-        }
+      if (format_flags & COLOR_ALPHA)
+        strcat (format_string, "A");
 
       switch (pt)
         {
@@ -584,12 +616,17 @@ query_exr (const gchar *path,
             strcat (format_string, " u32");
             break;
           case HALF:
+	    format_flags |= COLOR_FP16;
+            strcat (format_string, " half");
+            break;
           case FLOAT:
           default:
             format_flags |= COLOR_FP32;
             strcat (format_string, " float");
             break;
         }
+
+      
     }
   catch (...)
     {
@@ -598,14 +635,14 @@ query_exr (const gchar *path,
     }
 
   *ff_ptr = format_flags;
-  *format = (void*)babl_format (format_string);
+  *format = (void*)babl_format_with_space (format_string, space);
   return TRUE;
 }
 
 static GeglRectangle
 get_bounding_box (GeglOperation *operation)
 {
-  GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties   *o = GEGL_PROPERTIES (operation);
   GeglRectangle result = {0, 0, 10, 10};
   gint          w, h, ff;
   gpointer      format;
@@ -626,7 +663,7 @@ process (GeglOperation       *operation,
          const GeglRectangle *result,
          int                  level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   gint        w,h,ff;
   gpointer    format;
   gboolean    ok;
@@ -653,7 +690,7 @@ get_cached_region (GeglOperation       *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationSourceClass *source_class;
@@ -671,7 +708,10 @@ gegl_chant_class_init (GeglChantClass *klass)
     "categories"  , "hidden",
     "description" , "EXR image loader.", NULL);
 
-  gegl_extension_handler_register (".exr", "gegl:exr-load");
+  gegl_operation_handlers_register_loader (
+    "image/x-exr", "gegl:exr-load");
+  gegl_operation_handlers_register_loader (
+    ".exr", "gegl:exr-load");
 }
 
 #endif
